<?php

use UnicaenCode\Form\ElementMaker;
use UnicaenCode\Util;

/**
 * @var $this       \Application\View\Renderer\PhpRenderer
 * @var $controller \Zend\Mvc\Controller\AbstractController
 * @var $container  \Interop\Container\ContainerInterface
 * @var $viewName   string
 * @var $viewFile   string
 */

?>
    <h1>Génération d'une Classmap pour optimiser l'autoload</h1>
<?php

$form = new \Zend\Form\Form();
$form->add(ElementMaker::selectModule(
    'module', 'Module concerné'
));
$form->add(ElementMaker::submit('generate', 'Générer la Classmap'));
$form->setData($controller->getRequest()->getPost());

Util::displayForm($form);

if ($controller->getRequest()->isPost() && $form->isValid()) {
    affModMap($form->get('module')->getValue());
}



function affModMap(string $moduleName)
{
    $sCodeGenerator = Util::codeGenerator();

    $params = [
        'template' => 'autoload_classmap',
        'filename' => 'autoload_classmap.php',
        'map'      => genModMap($moduleName),
    ];

    ?>
    <div class="alert alert-info">Le fichier est récupérable dans le
        dossier <?php echo $sCodeGenerator->getOutputDir() ?></div>

    <h3>Etape 3 : Déclaration dans le fichier de configuration</h3>
    <?php $sCodeGenerator->generateFile($params, true); ?>
    <div class="alert alert-warning">
        Vous devez vous-même placer ces informations dans le fichier de configuration de votre
        module.
    </div>

    <?php
}



function genModMap(string $moduleName)
{
    $sIntrospection = Util::introspection();

    $modules = $sIntrospection->getModules(true);
    if (!isset($modules[$moduleName])) {
        throw new \Exception('Le module "' . $moduleName . '" est introuvable');
    }

    $module = $modules[$moduleName];

    $l = new \Zend\File\ClassFileLocator($module['path']);

    $mdl = strlen($module['path']);
    $map = new stdClass;
    foreach ($l as $file) {
        $filename = $file->getPath() .'/'. $file->getFilename();
        $filename = substr($filename, $mdl);

        foreach ($file->getClasses() as $class) {
            $map->{$class} = $filename;
        }
    }

    $php = '';
    $map = (array)$map;
    ksort($map);

    $mc = 0;
    foreach ($map as $c => $f) {
        $lc = strlen($c);
        if ($lc > $mc) $mc = $lc;
    }
    foreach ($map as $c => $f) {
        $php .= "    " . str_pad("'$c'", $mc + 2, ' ') . " => __DIR__ . '$f',\n";
    }

    return trim($php);
}