<?php

namespace UnicaenCode\Controller;

use UnicaenCode\Service\Traits\ConfigServiceAwareTrait;
use UnicaenCode\Util;
use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;
use Zend\Console\Request as ConsoleRequest;

class Controller extends AbstractActionController
{
    use ConfigServiceAwareTrait;

    public function indexAction()
    {
        $viewName  = $this->params()->fromRoute('view');

        $viewModel = new ViewModel;
        $viewModel->setTemplate('unicaen-code/index');
        $viewModel->setVariables([
            'controller' => $this,
            'container'  => Util::getContainer(),
            'viewName'   => $viewName,
            'viewFile'   => $this->getViewFile($viewName),
        ]);

        return $viewModel;
    }



    public function consoleAction()
    {
        $viewName = $this->getRequest()->getParam('script');
        $viewFile = $this->getViewFile($viewName);

        if ($viewFile){
            $controller = $this;
            $container = Util::getContainer();
            list($params,$variables) = $this->paramsFromRequest();
            extract($variables);

            include_once($viewFile);
        }else{
            throw new \Exception('Script '.$viewName.' Introuvable');
        }
    }



    private function paramsFromRequest()
    {
        $params = [];
        $variables = [];

        /* @var $request ConsoleRequest */
        $request = $this->getRequest();
        for( $i=0;$i<=9;$i++){
            if ($value = $request->getParam('p'.$i)){
                if (false !== ($eq=strpos($value,'='))){
                    $variables[substr($value, 0, $eq )] = substr($value, $eq+1 );
                }else{
                    $params[] = $value;
                }
            }
        }

        return [$params,$variables];
    }



    private function getViewFile($viewName)
    {
        $viewDirs = $this->getServiceConfig()->getViewDirs();
        $viewFile = null;
        foreach( $viewDirs as $viewDir ){
            if (file_exists($viewDir.'/'.$viewName.'.php')){
                $viewFile = $viewDir.'/'.$viewName.'.php';
                break;
            }
        }

        return $viewFile;
    }
}